function [V, kOpt] = Vn(k, state, Vpre, param, kOpt)
% function V = Vn(k, state, Vpre, param)
%
% maximizes V in an iterative way V=V_n and Vpre = V_n-1
%
% k, state:  vectors with the same length with the capital and 
%            state to calculate V_n(k,s)
% Vpre:      V from the last iteration
% param:     structure with the parameters of the model
% kOpt:     Is the optimal investment k(t+1) to every k(t), state(t)


    % Consistency Checks
    % ------------------
    
    if ~param.checked
        param = checkParam(param);
    end
        
    if nargin < 4
        error('Not enough input arguments');
    end
    
    if any(size(k)~=size(state))
        error('s and k need to have the same size');
    end
    
    if numel(k) ==1
       s = param.s;
        
        % Get starting values and boundary condtions for Maximization
        % ------------------------------------------------------------
        
             
        if nargin<5 | numel(kOpt)~=1 
            startK = k;
        else
            startK = kOpt;
        end
        
        % Investor cannot take debts or shortsell
        LB = 0; UB = 1;
        
        % Find Starting Value also in the case of multiple maximas
        % --------------------------------------------------------
        
        ff = f(k, s(state), param);
        if ff ~= 0
            startLambda = (LB:(UB-LB)/20:UB);
            startLambda = [startK/ff, startLambda];
            startK = startLambda * ff;
            temp = calcV(startK, k, state, Vpre, param);
            startLambda = startLambda(find(temp>=max(temp), 1, 'first'));
            
            
            
           % Calculate the maximum
           % ---------------------
           checkLocal = true; 
           if startLambda^2 <= 20*eps()
               check = [0; 1e-12; 1e-10; 1e-5; 1e-2; 4e-2];
               temp = calcV(check * ff, k, state, Vpre, param);             
               
               if all( (((temp(1)-temp(2:end))) / max(0.1, temp(1)) )> -1e-4 * check(2:end)) %omit errors due to nummerical issues
                   V = temp(1);
                   lambda = 0;
                   checkLocal = false;
               else
                   startLambda = check(find(temp>=max(temp), 1, 'last'));
               end
               
           elseif (startLambda-1)^2 <= 20*eps()
               check = 1-[0 ; 1e-12; 1e-10; 1e-5; 1e-2; 1e-1];
               temp = calcV(check * ff, k, state, Vpre, param);             
               
               %plot(check, temp)
               
               if all( (((temp(1)-temp(2:end))) / max(0.1, temp(1)) )> -1e-4 * check(2:end)) %omit errors due to nummerical issues
                   V = temp(1);
                   lambda = 1;
                   checkLocal = false;
                else
                   startLambda = check(find(temp>=max(temp), 1, 'last'));
               end
           end
                      
           if checkLocal
                options = optimset('Display', 'off');
                    warning off all
                [lambda, V] = fmincon(@(lambda) (calcV(lambda*ff, k, state, Vpre, param))*(-1), ...
                    startLambda, [],[],[],[],LB, UB, [], options);
        
                V = -V; 
           end
        else % if no production the solution is trivial
            lambda = 0;
            V =  calcV(0, k, state, Vpre, param);           
        end
            
  
        % Calc kOpt from lambda
        % sometimes the optimizer optimizes out of bounds
        % correction of that
        % ------------------------------------------------
        
        if lambda <LB
            kOpt = LB * ff;
            V    = calcV(LB, k, state, Vpre, param);
            
        elseif lambda > UB
            kOpt = UB * ff;
            V    = calcV(UB, k, state, Vpre, param);
        else
            kOpt = lambda * ff;
        end
        
       
        warning on all
        
    elseif numel(k) == 0
        V=[];
        if nargout == 2
            kOpt = [];
        end
    else

        % for more than one k and s calculate recursively
        % -----------------------------------------------
        
        V = zeros(size(k));
        if nargout == 2
            kOpt = NaN(size(k));
        end
        kStart = k(1);
        for ii=1:numel(k)               
           [V(ii), kStart] = Vn(k(ii), state(ii), Vpre, param, kStart);            
           if nargout == 2
                kOpt(ii) = kStart;
           end
        end
    end
    