function  V = calcV(kNext, k, state, Vpre, param)
% function  V = calcV(kNext, k (scalar), state (scalar), Vpre, param)
%
% evaluate the value of V given k(t+1) = kNext, k(t)=k and the state
% kNext can be given to the function as a vector
% note that the state has to be an integer referring to the
% shock in the state in the variable param.s
% in the first collumn of Vpre are the ks
% in the next s collums are the Vs for every state s
% every state is in a different collumn
    

    % Initialization and Consistency Checks
    % --------------------------------------

    oldSizeKNext = size(kNext);
    n            = numel(kNext);    
    kNext = reshape(kNext, n, 1);   
    
    if ~param.checked
        param = checkParam(param);
    end
    
    if numel(param.beta)==1
        beta = param.beta;
    else
        beta = param.beta(state);
    end
    
    p = param.p(:, state);
    s = param.s;
    scale = param.scale;
    nOfStates = numel(p);
    
    if numel(state)~=1
        error('To many states');
    end
    
    if fix(state) ~= state | any(state>nOfStates)
        error('state has to be an integer or the state is bigger than the number of existing states');
    end
    
    if numel(p)~= (size(Vpre, 2)-1)
        error('p or Vpre have a wrong dimension');
    end
    
    if numel(k) ~= 1 | numel(state)~=1
        error('k and state have to be scalars');
    end
    
    kMax = Vpre(end, 1);
    smallerKmax = kMax>kNext;
    
    if any(kNext<0)
        warning('kNext should be positive');
    end
    
    kNext(kNext<0) = 0;
    
    try
        kDefault = param.kDefault;
    catch
        % find points where companies default
        kDefault = NaN(3, nOfStates);
        for ii=1:nOfStates
           kDefault(1, ii) = fzero(@(kk) f(kk, 0, param) - s(ii), s(ii));
           kDefault(2, ii) = max(find(Vpre(:,1)<= kDefault(1, ii)));
           kDefault(3, ii) = min(find(Vpre(:,1)>= kDefault(1, ii)));
        end        
    end
    
    
    % calculate d_t
    % -------------
    
    V = f(k, s(state), param)-kNext;
    
    if any(V<0)
        warning('negative dividends');
        % V(V<0) = V(V<0) * 1e30;
    end
    
    % Calculate beta*E[V]
    % -------------------
    
    temp = zeros(n, nOfStates);
          
    % interpolate Vpre for all states
    if any(smallerKmax)
        pos = kNext(smallerKmax).^(1/scale) - Vpre(1,1).^(1/scale);
        delta = (Vpre(end,1).^(1/scale) - Vpre(1,1).^(1/scale)) ...
            / (size(Vpre, 1)-1);
        pos = floor(1+ pos ./ delta);
        d   = diff(Vpre(:,1));
        d   = (kNext(smallerKmax)-Vpre(pos,1)) ./ d(pos);     
    
        temp(smallerKmax,:) = (Vpre(1+pos, 1+(1:nOfStates)) - Vpre(pos, 1+(1:nOfStates)));
 
        for ii = 1:nOfStates
            temp(smallerKmax,ii) = temp(smallerKmax,ii) .* d;
        end
        % temp(smallerKmax,:) = bsxfun(@times, temp(smallerKmax,:), d); % Much to slow
        temp(smallerKmax,:) = Vpre(pos, 1+(1:nOfStates)) + temp(smallerKmax,:);
    end
        
%        Without loop
%        temp(smallerKmax,:) = Vpre(pos, 1+(1:nOfStates)) + repmat(d, 1, nOfStates) .* ...
%             (Vpre(1+pos, 1+(1:nOfStates)) - Vpre(pos, 1+(1:nOfStates)));
        
%       interpolation does only work within the defined k
%       therefore check if kNext is outside of the range 
%       of Vpre(:, 1)
        



    % Assume if kNext is out of range that the slope of V is 1
    nToBig = sum(~smallerKmax);
    
    if nToBig>0
        temp(~smallerKmax,:) = repmat(Vpre(end, 1+(1:nOfStates) ),nToBig,1) ...
            +repmat(kNext(~smallerKmax)-kMax, 1, nOfStates); 
    end
    % it is assumed that V is 0 in the negative area.
    temp(kNext<0, :) = 0; % in this area the interpolation does not work
    if any(isnan(temp)) 
        error('variable to interpolate is out of range');
    end
    
    % Interpolate differently around default
    uB = Vpre(kDefault(3,:),1);
    lB = Vpre(kDefault(2,:),1);
    tP = kDefault(1,:)'; 
    defaulted = k>lB & k <uB ;
    
    if any(defaulted)
       lV = Vpre(kDefault(2,:)',2:end);
       uV = Vpre(kDefault(3,:)',2:end);
       temp( defaulted & k<=tP) = lV( defaulted & k<=tP);
       oTP = defaulted & k>=tP;
       if any(oTP)
           temp(oTP) = lV(oTP) + (uV(oTP)-lV(oTP))./ (uB(oTP)-tP(oTP)) .*(uB(oTP)-k); 
       end
    end
    
    

    % calculate conditional expected value
    temp  = temp*p;
    
    % Calculate V
    % ------------
    V = V + beta.*temp;
    V = reshape(V, oldSizeKNext); % undo vectorizing of kNext
    